package jms.client;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.Queue;
import javax.jms.QueueSender;
import javax.jms.QueueReceiver;
import javax.jms.DeliveryMode;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.jms.MessageListener;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;


public class SendAndReceive
{
    private String          requestFactory;
    private String          requestDestination;
    private String          responseFactory;
    private String          responseDestination;
    private QueueConnection requestConnection;
    private QueueSession    requestSession;
    private QueueConnection responseConnection;
    private QueueSession    responseSession;
    private QueueSender     requestSender;
    private QueueReceiver   responseReceiver;


    public SendAndReceive (String requestFactory, String requestDestination,
                           String responseFactory,String responseDestination)
    {
        this.requestFactory = requestFactory;
        this.requestDestination = requestDestination;
        this.responseFactory = responseFactory;
        this.responseDestination = responseDestination;
    }

    public void initialize ()
        throws JMSException, NamingException
    {
        QueueConnectionFactory reqqcf, resqcf;

        InitialContext iniCtx = new InitialContext ();

        reqqcf = (QueueConnectionFactory) iniCtx.lookup (requestFactory);
        resqcf = (QueueConnectionFactory) iniCtx.lookup (responseFactory);

        Queue reqDestination = (Queue) iniCtx.lookup (requestDestination);
        Queue resDestination = (Queue) iniCtx.lookup (responseDestination);

        requestConnection  = reqqcf.createQueueConnection ();
        responseConnection = resqcf.createQueueConnection ();

        requestSession  = requestConnection.createQueueSession (false,
                                                                QueueSession.AUTO_ACKNOWLEDGE);
        responseSession = responseConnection.createQueueSession (false,
                                                                 QueueSession.AUTO_ACKNOWLEDGE);

        requestSender    = requestSession.createSender   (reqDestination); 
        responseReceiver = responseSession.createReceiver (resDestination); 

        responseConnection.start();
    }


    public void terminate () throws JMSException
    {
        responseConnection.stop ();
        requestSender.close ();
        responseReceiver.close ();
        requestSession.close ();
        responseSession.close ();
        requestConnection.close ();
        responseConnection.close ();
    }


    public void run () throws JMSException
    {
        int messageCount = 5;

        // Let's send the messages.
        System.out.println ("========== Send Messages =================");
        for (int i=0; i<messageCount; i++) {
            TextMessage request = requestSession.createTextMessage ();
            request.setText ("Message " + i);
            requestSender.send (request, DeliveryMode.NON_PERSISTENT, 4, 0);
            System.out.println ("Message sent: " + request.getText ());
        }
        // Let's receive the messages.
        System.out.println ("========== Receive Messages ==============");
        while (true) {
            TextMessage response = (TextMessage)responseReceiver.receive (10000);
            if (response == null) {
                // We reach the timeout. Probably no more messages.
                break;
            }
            System.out.println ("Message received: " + response.getText ());
        }
    }

    public static void main (String[] args)
    {
        SendAndReceive sar = new SendAndReceive (args [0], args [1],
                                                 args [2], args [3]);
        try {
            sar.initialize ();
            sar.run ();
            sar.terminate ();
        }
        catch (JMSException jmsex) {
            jmsex.printStackTrace ();
        }
        catch (NamingException nex) {
            nex.printStackTrace ();
        }
    }
}
