package jms.server;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.EJBException;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueSender;
import javax.jms.Queue;
import javax.jms.JMSException;

import java.util.Date;
import java.util.Random;

public class MDB implements MessageDrivenBean, MessageListener {

    private MessageDrivenContext mdbContext = null;

    private String          responseFactoryName     = null;
    private String          responseDestinationName = null;
    private Queue           responseDestination     = null;
    private QueueConnection responseConnection      = null;
    private boolean         responseTransacted      = false;
    private boolean         containerTransaction    = false;

      
    private int             msgCount        = 0;
    private Random          random          = null;


    public void setMessageDrivenContext(MessageDrivenContext c)
        throws EJBException 
    {
        mdbContext = c;
    }


    public void ejbCreate()
    {
        System.out.println ("MDB::ejbCreate - begin");
        try {
            InitialContext ctx = new InitialContext ();

            responseFactoryName     = (String)ctx.lookup("java:comp/env/mdb/responseFactory");
            responseDestinationName = (String)ctx.lookup("java:comp/env/mdb/responseDestination");
            responseTransacted      = ((Boolean)ctx.lookup("java:comp/env/mdb/responseTransacted")).booleanValue ();
            containerTransaction    = ((Boolean)ctx.lookup("java:comp/env/mdb/containerTransaction")).booleanValue ();

            QueueConnectionFactory qcf;
            qcf = (QueueConnectionFactory) ctx.lookup (responseFactoryName);
            responseConnection  = qcf.createQueueConnection ();

            responseDestination = (Queue) ctx.lookup (responseDestinationName);

            random = new Random (new Date ().getTime ());
        }
        catch (JMSException jmsex) {
            throw (new EJBException (jmsex));
        }
        catch (NamingException nex) {
            throw (new EJBException (nex));
        }
        System.out.println ("MDB::ejbCreate - end");
    }

    
    public void ejbRemove()
    {
        System.out.println ("MDB::ejbRemove - begin");
        try {
            if (responseConnection != null) {
                responseConnection.close ();
            }
        }
        catch (JMSException jmsex) {
            // ignore
        }
        System.out.println ("MDB::ejbRemove - end");
    }

    
    public void onMessage (Message message)
    {
        System.out.println ("MDB::onMessage - begin");

        QueueSession responseSession = null;
        msgCount++;

        try {
            String req = ((TextMessage)message).getText();

            String res = businessLogic (req);

            responseSession = 
                responseConnection.createQueueSession (responseTransacted,
                                                       QueueSession.AUTO_ACKNOWLEDGE);
            QueueSender responseSender = responseSession.createSender (responseDestination);

            TextMessage responseMessage = responseSession.createTextMessage ();
            responseMessage.setText (res);

            responseSender.send (responseMessage);

            if (containerTransaction) {
                simulatePossibleFailure (req);
            }
        }
        catch (JMSException jmsex) {
            System.out.println ("MDB::onMessage: JMS Exception occured:");
            jmsex.printStackTrace ();
            Exception lex = jmsex.getLinkedException ();
            if (lex!=null) {
                System.out.println ("MDB::onMessage: JMS Linked Exception:");
                System.err.println ("========================");
                lex.printStackTrace ();
            }
            if (containerTransaction) {
                mdbContext.setRollbackOnly ();
            }
        }
        catch (Exception ex) {
            ex.printStackTrace ();
            throw (new EJBException (ex));
        }
        finally {
            // ALWAYS close the session. It's pooled, so do not worry.
            if (responseSession != null) {
                try {
                    responseSession.close();
                }
                catch (Exception e) {
                    // Ignore
                    //e.printStackTrace();
                }
            }
        }
        System.out.println ("MDB::onMessage - end");    
    }


    private String businessLogic (String message)
    {
        System.out.println("Got Message: " + message);
        return ("Response to " + message);
    }


    private void simulatePossibleFailure (String message)
    {
        if (random.nextInt (30) > 10) {
            System.out.println("Abort: " + message);
            mdbContext.setRollbackOnly ();
        }
    }
}
