package jmx.service.wsmq;

import javax.jms.JMSException;

import com.ibm.mq.jms.MQConnectionFactory;
import com.ibm.mq.jms.MQQueueConnectionFactory;
import com.ibm.mq.jms.MQXAQueueConnectionFactory;
import com.ibm.mq.jms.MQTopicConnectionFactory;
import com.ibm.mq.jms.MQXATopicConnectionFactory;
import com.ibm.mq.jms.JMSC;

import jmx.service.jms.JMS;

/**
 * MBean implementation to add a WebSphere MQ connection factory in the JNDI namespace.
 * Check WSMQ JMS classes documentation for more details.
 */
public class WSMQConnectionFactory extends JMS implements WSMQConnectionFactoryMBean
{
    private int     _ccsid            = 1208; //UTF8
    private String  _channel          = null;
    private int     _fiqValue         = JMSC.MQJMS_FIQ_YES;
    private String  _hostName         = "localhost";
    private int     _port             = 1414;
    private String  _queueManagerName = null;
    private int     _transportType    = JMSC.MQJMS_TP_CLIENT_MQ_TCPIP;

    public int getCCSID ()
    {
        return _ccsid;
    }

    public void setCCSID (int ccsid)
    {
        _ccsid = ccsid;
    }

    public String getChannel ()
    {
        return _channel;
    }

    public void setChannel (String channel)
    {
        _channel = channel;
    }

    public boolean getFailIfQuiesce ()
    {
        if (_fiqValue == JMSC.MQJMS_FIQ_YES) {
            return true;
        }
        else {
            return false;
        }
    }
    
    public void setFailIfQuiesce (boolean fiqValue)
    {
        if (fiqValue) {
            _fiqValue = JMSC.MQJMS_FIQ_YES;
        }
        else {
            _fiqValue = JMSC.MQJMS_FIQ_NO;
        }
    }

    public String getHostName ()
    {
        return _hostName;
    }

    public void setHostName (String hostName)
    {
        _hostName = hostName;
    }

    public int getPort ()
    {
        return _port;
    }

    public void setPort (int port)
    {
        _port = port;
    }

    public String getQueueManagerName ()
    {
        return _queueManagerName;
    }

    public void setQueueManagerName(String queueManagerName)
    {
        _queueManagerName = queueManagerName;
    }

    public String getTransportType ()
    {
        switch (_transportType) {
        case JMSC.MQJMS_TP_BINDINGS_MQ:
            return "MQJMS_TP_BINDINGS_MQ";
        case JMSC.MQJMS_TP_CLIENT_MQ_TCPIP:
        default:
            return "JMSC.MQJMS_TP_CLIENT_MQ_TCPIP";
        case JMSC.MQJMS_TP_DIRECT_TCPIP:
            return "MQJMS_TP_DIRECT_TCPIP";
        }
    }
    
    public void setTransportType (String transportType)
    {
        if (transportType.equals ("MQJMS_TP_BINDINGS_MQ")) {
            _transportType = JMSC.MQJMS_TP_BINDINGS_MQ;
        }
        else if (transportType.equals ("MQJMS_TP_CLIENT_MQ_TCPIP")) {
            _transportType = JMSC.MQJMS_TP_CLIENT_MQ_TCPIP;
        }
        else if (transportType.equals ("MQJMS_TP_DIRECT_TCPIP")) {
            _transportType = JMSC.MQJMS_TP_DIRECT_TCPIP;
        }
        else {
            _transportType = JMSC.MQJMS_TP_CLIENT_MQ_TCPIP;
        }
    }

    /**
     * Creates the WSMQ connection factory object.
     */
    public Object createObject () throws JMSException
    {
        MQConnectionFactory cf = null;

        if (getJMSStyle ().equals (TOPIC_JMS_STYLE)) {
            if (getIsXA ()) {
                cf = new MQXATopicConnectionFactory ();
            }
            else {
                cf = new MQTopicConnectionFactory ();
            }
        }
        else {
            if (getIsXA ()) {
                cf = new MQXAQueueConnectionFactory ();
            }
            else {
                cf = new MQQueueConnectionFactory ();
            }
        }

        cf.setCCSID (_ccsid);
        if (_channel != null) {
            cf.setChannel (_channel);
        }
        cf.setFailIfQuiesce(_fiqValue);
        if (_hostName != null) {
            cf.setHostName (_hostName);
        }
        cf.setPort (_port);
        cf.setQueueManager (_queueManagerName);
        cf.setTransportType (_transportType);

        return cf;
    }
}


