package jmx.service.wsmq;

import javax.jms.JMSException;

import com.ibm.mq.jms.MQDestination;
import com.ibm.mq.jms.MQQueue;
import com.ibm.mq.jms.MQTopic;
import com.ibm.mq.jms.JMSC;

import jmx.service.jms.JMS;

/**
 * MBean implementation to add a WebSphere MQ destination in the JNDI namespace.
 * Check WSMQ JMS classes documentation for more details.
 */
public class WSMQDestination extends JMS implements WSMQDestinationMBean
{
    private String  _queueManagerName = null;
    private String  _destinationName  = null;;
    private int     _ccsid            = 1208; //UTF8
    private String  _description      = null;
    private int     _encoding         = JMSC.MQJMS_ENCODING_NATIVE;
    private int     _expiry           = JMSC.MQJMS_EXP_APP;
    private int     _fiqValue         = JMSC.MQJMS_FIQ_YES;
    private int     _persistence      = JMSC.MQJMS_PER_APP;
    private int     _priority         = JMSC.MQJMS_PRI_APP;
    private int     _targetClient     = JMSC.MQJMS_CLIENT_JMS_COMPLIANT;

    public String getQueueManagerName ()
    {
        return _queueManagerName;
    }

    public void setQueueManagerName (String queueManagerName)
    {
        _queueManagerName = queueManagerName;
    }

    public String getDestinationName ()
    {
        return _destinationName;
    }

    public void setDestinationName (String destinationName)
    {
        _destinationName = destinationName;
    }

    public int getCCSID ()
    {
        return _ccsid;
    }

    public void setCCSID (int ccsid)
    {
        _ccsid = ccsid;
    }

    public String getDescription ()
    {
        return _description;
    }

    public void setDescription (String description)
    {
        _description = description;
    }

    public String getEncoding ()
    {
        // Make it more user friendly than just a number.
        return getStringFromEncoding (_encoding);
    }

    public void setEncoding (String encoding)
    {
        // Make it more user friendly than just a number.
        _encoding = getEncodingFromString (encoding);
    }

    public int getExpiry ()
    {
        return _expiry;
    }

    public void setExpiry (int expiry)
    {
        _expiry = expiry;
    }

    public boolean getFailIfQuiesce ()
    {
        if (_fiqValue == JMSC.MQJMS_FIQ_YES) {
            return true;
        }
        else {
            return false;
        }
    }
    
    public void setFailIfQuiesce (boolean fiqValue)
    {
        if (fiqValue) {
            _fiqValue = JMSC.MQJMS_FIQ_YES;
        }
        else {
            _fiqValue = JMSC.MQJMS_FIQ_NO;
        }
    }

    public String getPersistence ()
    {
        // Make it more user friendly than just a number.
        return getStringFromPersistence (_persistence);
    }

    public void setPersistence (String persistence)
    {
        // Make it more user friendly than just a number.
        _persistence = getPersistenceFromString (persistence);
    }

    public int getPriority ()
    {
        return _priority;
    }

    // -2 for MQJMS_PRI_APP, -1 for MQJMS_PRI_QDEF
    public void setPriority (int priority)
    {
        _priority = priority;
    }

    public String getTargetClient ()
    {
        // Make it more user friendly than just a number.
        if (_targetClient == JMSC.MQJMS_CLIENT_JMS_COMPLIANT) {
            return "MQJMS_CLIENT_JMS_COMPLIANT";
        }
        else {
            return "MQJMS_CLIENT_NONJMS_MQ";
        }
    }

    public void setTargetClient (String targetClient)
    {
        // Make it more user friendly than just a number.
        if (targetClient == "MQJMS_CLIENT_JMS_COMPLIANT") {
            _targetClient = JMSC.MQJMS_CLIENT_JMS_COMPLIANT;
        }
        else {
            _targetClient = JMSC.MQJMS_CLIENT_NONJMS_MQ;
        }
    }

    /**
     * Creates the WSMQ destination object.
     */
    public Object createObject () throws Exception
    {
        MQDestination dest = null;

        if (getJMSStyle ().equals (TOPIC_JMS_STYLE)) {
            dest = new MQTopic ();
            ((MQTopic)dest).setBaseTopicName (_destinationName);
        }
        else {
            dest = new MQQueue ();
            ((MQQueue)dest).setBaseQueueManagerName (_queueManagerName);
            ((MQQueue)dest).setBaseQueueName (_destinationName);
        }

        dest.setCCSID (_ccsid);
        dest.setDescription (_description);
        dest.setEncoding (_encoding);
        dest.setExpiry (_expiry);
        dest.setFailIfQuiesce (_fiqValue);
        dest.setPersistence (_persistence);
        dest.setPriority (_priority);
        dest.setTargetClient (_targetClient);

        return dest;
    }

    private String getStringFromEncoding (int encoding)
    {
        switch (encoding) {
        case JMSC.MQJMS_ENCODING_INTEGER_NORMAL:
            return "MQJMS_ENCODING_INTEGER_NORMAL";
        case JMSC.MQJMS_ENCODING_INTEGER_REVERSED:
            return "MQJMS_ENCODING_INTEGER_REVERSED";
        case JMSC.MQJMS_ENCODING_DECIMAL_NORMAL:
            return "MQJMS_ENCODING_DECIMAL_NORMAL";
        case JMSC.MQJMS_ENCODING_DECIMAL_REVERSED:
            return "MQJMS_ENCODING_DECIMAL_REVERSED";
        case JMSC.MQJMS_ENCODING_FLOAT_IEEE_NORMAL:
            return "MQJMS_ENCODING_FLOAT_IEEE_NORMAL";
        case JMSC.MQJMS_ENCODING_FLOAT_IEEE_REVERSED:
            return "MQJMS_ENCODING_FLOAT_IEEE_REVERSED";
        case JMSC.MQJMS_ENCODING_FLOAT_S390:
            return "MQJMS_ENCODING_FLOAT_S390";
        case JMSC.MQJMS_ENCODING_NATIVE:
        default:
            return "MQJMS_ENCODING_NATIVE";
        }
    } 

    private int getEncodingFromString (String encoding)
    {
        if (encoding.equals ("MQJMS_ENCODING_INTEGER_NORMAL")) {
            return JMSC.MQJMS_ENCODING_INTEGER_NORMAL;
        }
        if (encoding.equals ("MQJMS_ENCODING_INTEGER_REVERSED")) {
            return JMSC.MQJMS_ENCODING_INTEGER_REVERSED;
        }
        if (encoding.equals ("MQJMS_ENCODING_DECIMAL_NORMAL")) {
            return JMSC.MQJMS_ENCODING_DECIMAL_NORMAL;
        }
        if (encoding.equals ("MQJMS_ENCODING_DECIMAL_REVERSED")) {
            return JMSC.MQJMS_ENCODING_DECIMAL_REVERSED;
        }
        if (encoding.equals ("MQJMS_ENCODING_DECIMAL_REVERSED")) {
            return JMSC.MQJMS_ENCODING_DECIMAL_REVERSED;
        }
        if (encoding.equals ("MQJMS_ENCODING_FLOAT_IEEE_NORMAL")) {
            return JMSC.MQJMS_ENCODING_FLOAT_IEEE_NORMAL;
        }
        if (encoding.equals ("MQJMS_ENCODING_FLOAT_IEEE_REVERSED")) {
            return JMSC.MQJMS_ENCODING_FLOAT_IEEE_REVERSED;
        }
        if (encoding.equals ("MQJMS_ENCODING_FLOAT_S390")) {
            return JMSC.MQJMS_ENCODING_FLOAT_S390;
        }
        if (encoding.equals ("MQJMS_ENCODING_NATIVE")) {
            return JMSC.MQJMS_ENCODING_NATIVE;
        }
        return JMSC.MQJMS_ENCODING_NATIVE;
    } 

    private String getStringFromPersistence (int persistence)
    {
        switch (persistence) {
        case JMSC.MQJMS_PER_QDEF:
            return "MQJMS_PER_QDEF";
        case JMSC.MQJMS_PER_PER:
            return "MQJMS_PER_PER";
        case JMSC.MQJMS_PER_NON:
            return "MQJMS_PER_NON";
        case JMSC.MQJMS_PER_APP:
        default:
            return "MQJMS_PER_APP";
        }
    } 

    private int getPersistenceFromString (String persistence)
    {
        // Accept the WSMQ JMS API constant names as well
        // as the standard JMS API ones.
        if (persistence.equals ("MQJMS_PER_APP")) {
            return JMSC.MQJMS_PER_APP;
        }
        if (persistence.equals ("MQJMS_PER_QDEF")) {
            return JMSC.MQJMS_PER_QDEF;
        }
        if (persistence.equals ("MQJMS_PER_PER") ||
            persistence.equals ("PERSISTENT")) {
            return JMSC.MQJMS_PER_PER;
        }
        if (persistence.equals ("MQJMS_PER_NON") ||
            persistence.equals ("NON_PERSISTENT")) {
            return JMSC.MQJMS_PER_NON;
        }
        return JMSC.MQJMS_PER_APP;
    } 
}
